// ShowMemC.cpp : Definiert den Einstiegspunkt fr die Anwendung.
//

#include "stdafx.h"
#include "ShowMemC.h"

// Funktion GlobalMemoryStatusEx fr dynamisches Linking definieren und Zeiger anlegen
typedef BOOL (CALLBACK* GlobalMemoryStatusExDynLink)(LPMEMORYSTATUSEX);
GlobalMemoryStatusExDynLink GlobalMemoryStatusExFunc = NULL;

BOOL MsgErr(HWND hWnd, DWORD dwCode, LPCTSTR lpTitle)
{
	TCHAR szBuf[1024];
    if(FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, dwCode, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR)&szBuf[0], 1024, NULL))
	{
		MessageBox(hWnd, (LPCTSTR)&szBuf[0], lpTitle, MB_ICONERROR);
		return TRUE;
	}
	else
	{
		wsprintf(szBuf, L"Error retrieving message for error code %d.", dwCode);
		MessageBox(hWnd, szBuf, lpTitle, MB_ICONERROR);
		return FALSE;
	}
}

TCHAR *FormatNumber(DWORDLONG uulNumber, TCHAR *lpBuffer)
{
	int i, cnt;
	TCHAR szBuf[20];
	TCHAR *lpBufPos;
	// Hinweis: Wenn GlobalMemoryStatusExFunc existiert mssen 64-Bit-Zahlen konvertiert werden. 
	// Wenn die Funktion nicht existiert, haben wir es nur mit 32-Bit-Zahlen zu tun. Das ist z. B. 
	// bei Windows NT 4.0 der Fall. Sinnigerweise kennt die Funktion wsprintf unter Windows NT 4.0 
	// die 64-Bit-Erweiterung bei den Zahlenformaten auch nicht, weshalb die Unterscheidung 
	// unbedingt notwendig ist.
	if(GlobalMemoryStatusExFunc) cnt = wsprintf(szBuf, L"%I64u", uulNumber);
	else cnt = wsprintf(szBuf, L"%u", (ULONG)uulNumber);
	lpBufPos = lpBuffer;
	for(i = 0; i < cnt; i++)
	{
		if((i > 0) && (((cnt - i) % 3) == 0))
		{
			lpBufPos[0] = L'.';
			lpBufPos++;
		}
		lpBufPos[0] = szBuf[i];
		lpBufPos++;
	}
	wsprintf(lpBufPos, L" KB", uulNumber);
	return lpBuffer;
}

BOOL ShowMemPair(HWND hDlg, DWORDLONG uulTotalMem, DWORDLONG uulAvailMem, int nIdStaticUsed, int nIdStaticFree)
{
	TCHAR szBuf[25];
	FormatNumber(((uulTotalMem - uulAvailMem) / 1024), szBuf);
	SetDlgItemText(hDlg, nIdStaticUsed, szBuf);
	FormatNumber((uulAvailMem / 1024), szBuf);
	SetDlgItemText(hDlg, nIdStaticFree, szBuf);
	return TRUE;
}

/// <summary> Bestimmt die aktuelle Speichernutzung mit GlobalMemoryStatus oder GlobalMemoryStatusEx. </summary>
/// <param name="lpBuffer"> Zeiger auf MEMORYSTATUSEX-Struktur, welche die Ergebnisse entgegen nimmt. </param>
/// <returns> TRUE, wenn der Aufruf erfolgreich war, sonst FALSE. Mit GetLastError lassen sich weitere Informationen abrufen. </returns>
/// <remarks> Da ltere Betriebssysteme in der kernel32.dll noch nicht ber die Funktion GlobalMemoryStatusEx verfgen 
/// wird diese Funktion in der Einstiegsprozedur dynamisch gelinkt. Klappt das nicht, wird statt dessen die Funktion 
/// GlobalMemoryStatus verwendet. </remarks>
/// <history>
/// [06/07/10 Bertel] Neu.
/// </history>
BOOL GetMemUse(LPMEMORYSTATUSEX lpBuffer)
{
	if(GlobalMemoryStatusExFunc)
	{
		return GlobalMemoryStatusExFunc(lpBuffer);
	}
	else
	{
		MEMORYSTATUS stat;
		stat.dwLength = sizeof(stat);
		GlobalMemoryStatus(&stat);
		lpBuffer->dwMemoryLoad = stat.dwMemoryLoad;
		lpBuffer->ullAvailPageFile = stat.dwAvailPageFile;
		lpBuffer->ullAvailPhys = stat.dwAvailPhys;
		lpBuffer->ullAvailVirtual = stat.dwAvailVirtual;
		lpBuffer->ullTotalPageFile = stat.dwTotalPageFile;
		lpBuffer->ullTotalPhys = stat.dwTotalPhys;
		lpBuffer->ullTotalVirtual = stat.dwTotalVirtual;
		return TRUE;
	}
}

BOOL ShowMem(HWND hDlg)
{
	MEMORYSTATUSEX stat;
	stat.dwLength = sizeof(stat);
	if(GetMemUse(&stat))
	{
		ShowMemPair(hDlg, stat.ullTotalPageFile, stat.ullAvailPageFile, IDC_STATIC_TMU, IDC_STATIC_TMF);
		ShowMemPair(hDlg, stat.ullTotalPhys, stat.ullAvailPhys, IDC_STATIC_PMU, IDC_STATIC_PMF);
		if(IsIconic(hDlg))
		{
			TCHAR szBuf[25];
			FormatNumber(((stat.ullTotalPageFile - stat.ullAvailPageFile) / 1024), szBuf);
			SetWindowText(hDlg, szBuf);
		}
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

LRESULT CALLBACK WndProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
	case WM_INITDIALOG:
		// Wenn GlobalMemoryStatusEx nicht geladen werden konnte, NT4-Modus anzeigen.
		if(!GlobalMemoryStatusExFunc) ShowWindow(GetDlgItem(hDlg, IDC_STATIC_NT4), SW_SHOW);
		ShowMem(hDlg);
		SetTimer(hDlg, 54826, 1000, NULL);
		SetClassLongPtr(hDlg, GCLP_HICON, (LONG_PTR)LoadIcon((HINSTANCE)lParam, MAKEINTRESOURCE(IDI_SHOWMEMC)));
		SetClassLongPtr(hDlg, GCLP_HICONSM, NULL);
		return TRUE;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDC_BTN_MINIMIZE:
			ShowWindow(hDlg, SW_MINIMIZE);
			return TRUE;
		case IDCANCEL:
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	case WM_TIMER:
		if(wParam == 54826)
		{
			ShowMem(hDlg);
		}
		break;
	case WM_SIZE:
		switch(wParam)
		{
		case SIZE_MINIMIZED:
			ShowMem(hDlg);
			break;
		case SIZE_RESTORED:
			SetWindowText(hDlg, L"Show Memory");
			ShowMem(hDlg);
			break;
		}
		break;
	}
	return FALSE;
}

typedef BOOL (CALLBACK* InitCommonControlsExDynLink)(LPINITCOMMONCONTROLSEX);

VOID WinMainEntry()
{
	UINT result = 0;
	// Instanzhandle selbst bestimmen
	HINSTANCE hInstance = GetModuleHandle(NULL);
	//HINSTANCE hPrevInstance;
	//LPTSTR lpCmdLine;
	//int nCmdShow;
	// [07/05/15 Bertel] Registering common controls to enable xp styles
	INITCOMMONCONTROLSEX icc;
	icc.dwSize = sizeof(INITCOMMONCONTROLSEX);
	icc.dwICC = ICC_STANDARD_CLASSES;
	if(InitCommonControlsEx(&icc))
	{
		// Load the dll and keep the handle to it
		// Damit es unter WinNT4 luft, wird absichtlich die Nicht-Unicode-Variante 
		// verwendet. Die Unicode-Variante LoadLibraryW (=LoadLibrary) ruft 
		// GetLongPathNameW auf, was es unter WinNT4 normalerweise nicht gibt.
		HINSTANCE kernelHandle = LoadLibraryA("kernel32.dll");
		// If the handle is valid, try to get the function address. 
		if(kernelHandle) 
		{ 
			//Get pointer to our function using GetProcAddress:
			GlobalMemoryStatusExFunc = (GlobalMemoryStatusExDynLink)GetProcAddress(kernelHandle, "GlobalMemoryStatusEx");

			if(DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_MAINWIN), 0, (DLGPROC)WndProc, (LPARAM)hInstance) < 0)
			{
				MsgErr(0, GetLastError(), L"Error opening dialog");
				result = (UINT)-2;
			}
			//Free the library:
			FreeLibrary(kernelHandle);       
		}
		else
		{
			MsgErr(0, GetLastError(), L"Error loading kernel32.dll");
			result = (UINT)-3;
		}
	}
	else
	{
		MsgErr(0, GetLastError(), L"Error registering common controls");
		result = (UINT)-4;
	}
	ExitProcess(result);
}
